#include "Autonomous.h"
#include <frc/smartdashboard/SmartDashboard.h>

const double kMotorSpeed = .3;
const double kLeftTurn = -90;
const double kRightTurn = 90;
const double kShootSpeed = .7;
const double kShooterSpeed = .75;
// Old Auto
/*const double kMoveToPowerPort = 101.75;                            // ONLY FOR STARTING RIGHT, NOT FOR LEFT AND CENTER. Move up to power port (Step 0)
const double kStepZeroDrive = 72;                                  // ONLY FOR STARTING CENTER AND LEFT, NOT FOR RIGHT. Moving up to the turn point (Step 0)
const double kStepFourDrive = (kMoveToPowerPort - kStepZeroDrive); // ONLY FOR STARTING CENTER AND LEFT, NOT FOR RIGHT. Moving up t o the wall (Step 0)
const double kStepTwoDriveLeft = 140;                              // Lining up with the power port if you start left (Step 2)
const double kStepTwoDriveCenter = 72;                             // Linging up with the power port if you start center (Step 2)
// const double kBackOffPowerPort = -160;
const double kBackOffPowerPort = -10.75; // (Step 6)
const double kTurnToTrench = 161.12;     // (Step 7)
const double kMoveToPanel = 194.4;
const double kTurnToPanel = (180 - kTurnToTrench);
const double kCollectPanelBalls = 108;
const double kLeaveControlPanel = (-(kCollectPanelBalls + 168));
const double kLineUpToPowerPort = 62.905; // Lines up to the Power Port after backing up from the control panel
const double kGoToPowerPort = (-26.68);*/

// Constants for starting at right wall (moveLeft())
const double kRightWallForwardZero = -6;
const double kRightWallTurnTwo = 51;
const double kRightWallForwardFour = -119.5;
const double kRightWallTurnSix = 39;
const double kRightWallForwardEight = -10;
// The following are for picking up trench balls after power port
const double kTrenchForwardZero = 10;
const double kTrenchTurnTwo = -39;
const double kTrenchForwardFour = 119.5;
const double kTrenchTurnSix = (-(kTrenchTurnTwo));
// The following are for picking up trench balls and shooting  (FBA - Five Ball Auto)
const double kFBAForwardTwo = 0; // Start moving to trench balls
const double kFBAForwardEight = 0; // Go backwards to line (shooting position)


Autonomous::Autonomous(Drive *drive, GamePiece *gamePiece)
{
    this->drive = drive;
    this->gamePiece = gamePiece;
}

void Autonomous::process()
{ // Checks what auto
    switch (currentAutoMode)
    {
    case DONOTHING:
        doNothing();
        break;
    case CROSS_PORT:
        crossLine();
        break;
    case DRIVE_RIGHT_MORE:
        driveRightMore();
        break;
    case SHOOT_FROM_LINE:
        shootLine();
        break;
    case JUST_LINE:
        justLine();
        break;
    case DRIVE_LEFT:
        driveLeft();
        break;
    case CROSS_PORT_TRENCH:
        crossLine();
        trenchBalls();
        break;
    case FIVE_BALL_AUTO:
        fiveBallAuto();
        break;
    
    case AUTO_AIM:
        autoAim();
        break;
    }
}

void Autonomous::reset(){
    step = 0;
    trenchStep = -1;
}

void Autonomous::sendModesToDashboard()
{
    char buffer[256] = "";

    handleDashboardString(DONOTHING, "doing nothing", buffer); //do nothing
    handleDashboardString(CROSS_PORT, "start aligned with prt, drive, shoot", buffer); //start aligned with port, drive, shoot
    handleDashboardString(DRIVE_RIGHT_MORE, "start left wall, drive, shoot", buffer); //start left wall, drive, shoot
    handleDashboardString(SHOOT_FROM_LINE, "start aligned with prt, shoot, drive to CTR", buffer); //start aligned with port, shoot, drive to CTR
    handleDashboardString(JUST_LINE, "drive to ctr", buffer); //drive to CTR
    handleDashboardString(DRIVE_LEFT, "start right wall, drive, shoot", buffer); //start right wall, drive, shoot
    // handleDashboardString(CROSS_PORT_TRENCH, "start aligned with prt, drive, shoot, pick trench balls", buffer); //start aligned with prt, drive, shoot, pick trench balls
    handleDashboardString(FIVE_BALL_AUTO, "start right wall, pick up balls and shoot", buffer); //start right wall, pick up balls and shoot
    handleDashboardString(AUTO_AIM, "auto aim", buffer);
    SmartDashboard::PutString("thunderdashboard_auto_list", buffer);
}

void Autonomous::getModeFromDashboard()
{
    int mode = SmartDashboard::GetNumber("Auto_Mode", 0);
    currentAutoMode = (AutoMode)mode;
}

void Autonomous::debug(Feedback *feedback)
{
    feedback->sendDouble("Auto", "step", step);
    feedback->sendDouble("Auto", "limelight state", state);
    feedback->sendDouble("Auto", "current_AutoMode", currentAutoMode);

}

void Autonomous::doNothing()
{
    // if it does nothing is it doing something or nothing?
    // don't actually do anything
}

void Autonomous::justLine(){
    if (step == 0)
    {
        drive->autoForward(3 * 12, kMotorSpeed);
        step++;
    }
    else if (step == 1)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
}

void Autonomous::crossLine()
{
    if (step == 0)
    {
        drive->autoForward(-105.5, kMotorSpeed);
        step++;
    }
    else if (step == 1)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if(step == 2){
        gamePiece->hood(Shooter::HoodPosition::PORT);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 3){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if (step == 4)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 5)
    {
        if(gamePiece->getCellCount() == 0){
        step++;
        trenchStep = 0;
        }
    }
}

void Autonomous::driveRight()
//i believe the distance from the center of the power port is 58" vertical, 75" horizontal. this is 1/2 the real field dimensions.
{
    if (step == 0)
    {
        drive->autoForward(-3, kMotorSpeed);
        step++;
    }
    else if (step == 1)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 2)
    {
        drive->autoPivot(-28, kMotorSpeed);
        step++;
    }
    else if (step == 3)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 4)
    {
        drive->autoForward(-180, kMotorSpeed);
        step++;
    }
    else if (step == 5)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 6)
    {
        drive->autoPivot(-80, kMotorSpeed);
        step++;
    }
    else if (step == 7)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 8)
    {
        drive->autoForward(-20, kMotorSpeed);
        step++;
    }
    else if (step == 9)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if(step == 10){
        gamePiece->hood(Shooter::HoodPosition::PORT);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 11){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if (step == 12)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 13)
    {
        if(gamePiece->getCellCount() == 0){
        step++;
        }
    }
}

void Autonomous::driveRightMore()
//58" vertical, 104.7"(104") horizontal from the center of the power port. 1/2 dimensions from real field
{
    if (step == 0)
    {
        drive->autoForward(-6, kMotorSpeed);
        step++;
    }
    else if (step == 1)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 2)
    {
        drive->autoPivot(-24.5, kMotorSpeed);
        step++;
    }
    else if (step == 3)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 4)
    {
        drive->autoForward(-176, kMotorSpeed);
        step++;
    }
    else if (step == 5)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 6)
    {
        drive->autoPivot(-80, kMotorSpeed);
        step++;
    }
    else if (step == 7)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if (step == 8)
    {
        drive->autoForward(-76, kMotorSpeed);
        step++;
    }
    else if (step == 9)
    {
        if (drive->autoComplete())
        {
            step++;
        }
    }
    else if(step == 10){
        gamePiece->hood(Shooter::HoodPosition::PORT);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 11){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if (step == 12)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 13)
    {
        if(gamePiece->getCellCount() == 0){
        step++;
        }
    }
}
void Autonomous::shoot()
{
    if(step == 0){
        gamePiece->hood(Shooter::HoodPosition::PORT);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 1){
        if(timer.Get() >= 1){
            step++;
        }
    }
    if (step == 2)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 3)
    {
        if(gamePiece->getCellCount() == 0){
        step++;
        }
    }
}

void Autonomous::shootLine(){
    if(step == 0){
        gamePiece->hood(Shooter::HoodPosition::LINE);
        step++;
        timer.Reset();
        timer.Start();
        gamePiece->setShooterPreheat(true);
    }
    else if(step == 1){
        if(timer.Get() >= 2.5){
            step++;
        }
    }
    else if(step == 2){
        gamePiece->activateShooter(true);
        step++;
    }
    if(step == 3){
        if(gamePiece->getCellCount() == 0){
            gamePiece->activateShooter(false);
            step++;
        }
    }
    else if(step == 4){
        gamePiece->setShooterPreheat(false);
        drive->autoForward(12*4, kMotorSpeed);
        step++;
    }
    else if(step == 5){
        if(drive->autoComplete()){
            step++;
        }
    }
}

void Autonomous::driveLeft()
{
    if(step == 0){
        drive->autoForward(kRightWallForwardZero,kMotorSpeed);
        step++;
    }
    else if(step == 1){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 2){
        drive->autoPivot(kRightWallTurnTwo, kMotorSpeed);
        step++;
    }
    else if(step == 3){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 4){
        drive->autoForward(kRightWallForwardFour, kMotorSpeed);
        step++;
    }
    else if(step == 5){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 6){
        drive->autoPivot(kRightWallTurnSix, kMotorSpeed);
        step++;
    }
    else if(step == 7){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 8){
        drive->autoForward(kRightWallForwardEight, kMotorSpeed);
        step++;
    }
    else if(step == 9){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 10){
        gamePiece->hood(Shooter::HoodPosition::LINE);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 11){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if(step == 12){
        gamePiece->activateShooter(true);
        step++;
    }
    else if(step == 13){
        if(gamePiece->getCellCount() == 0){
            gamePiece->activateShooter(false);
            step++;
        }
    }
}
void Autonomous::trenchBalls()
{
    if(trenchStep == 0){
        drive->autoForward(kTrenchForwardZero, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 1){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 2){
        drive->autoPivot(kTrenchTurnTwo, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 3){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 4){
        drive->autoForward(kTrenchForwardFour, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 5){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 6){
        drive->autoPivot(kTrenchTurnSix, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 7){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 8){
        gamePiece->setIntakeState(GamePiece::INTAKE);
        trenchStep++;
    }
    else if(trenchStep == 9){
        drive->autoForward(216, kMotorSpeed);
        trenchStep++;
    }
    else if(trenchStep == 10){
        if(drive->autoComplete()){
            trenchStep++;
        }
    }
    else if(trenchStep == 11){
        gamePiece->setIntakeState(GamePiece::NOTTAKE);
        trenchStep++;
    }
}

void Autonomous::fiveBallAuto()
{
    
    if(step == 0){
        gamePiece->setIntakeState(GamePiece::INTAKE);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 1){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if(step == 2){
        drive->autoForward(kFBAForwardTwo, kMotorSpeed);
        step++;
    }
    else if(step == 3){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 4){
        gamePiece->setIntakeState(GamePiece::NOTTAKE);
        step++;
        timer.Reset();
        timer.Start();
    }
    else if(step == 5){
        if(timer.Get() >= .5){
            step++;
        }
    }
    else if(step == 6){
        drive->autoForward(kFBAForwardEight, kMotorSpeed);
        step++;
    }
    else if(step == 7){
        if(drive->autoComplete()){
            step++;
        }
    }
    else if(step == 8){
        timer.Reset();
        timer.Start();
        step++;
        autoAim();
    }
    else if(step == 9){
        if(timer.Get() >= 1){
            step++;
        }
    }
    else if (step == 10)
    {
        gamePiece->activateShooter(true);
        step++;
    }
    else if (step == 11)
    {
        if(gamePiece->getCellCount() == 0){
            step++;
            gamePiece->activateShooter(false);
        }
    }
}

void Autonomous::handleDashboardString(AutoMode mode, const char *description, char *buffer)
{
    char key[32];

    sprintf(&buffer[strlen(buffer)], ",%d", mode);
    sprintf(key, "thunderdashboard_auto_%d", mode);
    SmartDashboard::PutString(key, description);
}

void Autonomous::doStartupThings()
{
    gamePiece->hood(Shooter::PORT);
    getModeFromDashboard();
}

void Autonomous::autoAim(){
    double TX = limelight.get("tx");

    frc::SmartDashboard::PutNumber("limelight_tx", TX);
    if(TX >= 3){
        drive->setSpeed(.1, -.1);
    }
    else if(TX <= -3){
        drive->setSpeed(-.1, .1);
    }
    else {
        drive->setSpeed(0, 0);
    }
    double TY = limelight.get("ty");

    frc::SmartDashboard::PutNumber("limelight_ty", TY);
    
    if(TY >= 3){
        gamePiece->hood(Shooter::LINE);
    }
    else if(TY <= -3){
        gamePiece->hood(Shooter::PANEL);
    }
    else {
        gamePiece->hood(Shooter::PORT);
    }
    limelight.set("ledMode", 3);

    if((TX <= .1 || TX >= -.1) && (TY <= .1 || TY >= -.1)){
        timer2.Reset();
        timer2.Start();
        state = AIMED;
        limelight.set("ledMode", 2);
        if(timer.Get() >= 1){
            limelight.set("ledMode", 3);
        }
    }
    else {
        state = NOT;
        limelight.set("ledMode", 3);
    }
}