#pragma once

#include "Feedback.h"
#include "Intake.h"
#include "Helix.h"
#include "Shooter.h"
#include "Lights.h"
#include <frc/Counter.h>
#include <frc/Timer.h>
#include "Log.h"

class GamePiece {
    public:
        GamePiece();

        void process();
        // Reset the gamepiece systems, setting ballcount
        // to the value given.  If the value is < 0, the existing
        // ball count is kept
        void reset(int ballCount);
        void debug(Feedback* feedback);

        void lights(Lights* lights);

        enum IntakeState{
            INTAKE, //deploy intake mech, spin intake in, spin helix up, stop spin and retract mech when @ 5balls
            OUTTAKE, //retract intake mech, spin outtake out, spin helix out
            NOTTAKE //retract intake mech, stop intake spin, stop helix spin when ball is at top of helix
        };

        enum ShooterState{
            NOTSHOOTING, //default state; flywheel off, primer wheel off
            WANTTOSHOOT, //warmup shooter flywheel, wait for ball to get to top of helix
            SHOOT //when ball @ stop of helix, stop helix, activate shooter primer wheel, wait for ball to be shot (shooterBeam triggered)
        };

        Intake* intake;
        Helix* helix;
        Shooter* shooter;

    private:
        bool setNottake(); //bool which nottakes

        frc::DigitalInput intakeBeam {DIO_INTAKE_BEAM};
        frc::DigitalInput helixTopBeam {DIO_TRANSITION_BEAM};
        frc::Counter shooterBeam {DIO_SHOOT_BEAM};

        IntakeState intakeState; // object for IntakeState setting
        ShooterState shooterState; //object for ShooterState setting

        bool goGoGadgetShooter; //keeps track of whether or not they want to shoot
        bool shooterWantsPreheat;
        bool lastIntakeValue;

        int currentCellCount; //how many cells we currently have in the robot. PUT ON DASHBOARD
        int cellsShot; //keeps tally of how many cells we've shot. used in the currentCellCount calculation

        frc::Timer brokenShotTimer; 
        frc::Timer keepIntakeAfterFive;

        bool intakeCounterBroken;
        bool transitionCounterBroken;
        bool shooterCounterBroken;
        bool didWeHaveFiveCells;

        bool ballWasShotLast = false;

    public:
        void setIntakeState(IntakeState intState); //sets the intake state
        void activateShooter(bool shooterActivate); //activates the shooter

        void setShooterPreheat(bool preheat); //function that gets the shooter wheel to speed
        void hood(Shooter::HoodPosition position); // sets hood to one of the preset positions
        void setShooterSpeed(double speed); // sets the speed of the shooter to a speed
  
        static const int BALL_COUNT_UNKNOWN = -1;
        int getCellCount();

        void pivotSensorBroken(bool broken1); // notifies that the pivot sensor is broken
        void setIntakeCounterBroken(bool broken2); // notifies that the intake ball senor is broken
        void setTransitionCounterBroken(bool broken3); // notifies that the shooter transition counter is broken
        void setShooterBallCounterBroken(bool broken4); // notifies that the shooter ball counter is broken
        void setIntakeIdleMode(bool enabled); //passes to intake

        void handleLog(Log* log);

};