#pragma once

#include "IOMap.h"
#include "ThunderSparkMax.h"
#include <frc/Servo.h>
#include <frc/DigitalInput.h>
#include "Feedback.h"


class Hang {
    public:
    Hang();

    enum HangMovement {
        UP, STOP, DOWN
    };

    void reset();
    void process();
    void debug(Feedback* feedback);

    /**
     *  Moves the mechanism to the target position 
     */
    void move(HangMovement direction);

    /**
     * When retracting, move slower for precision
     */
    void enableSlowRetract(bool enable);

    /**
     * broke is true if the zero sensor is broken
     * broke is false if the zero sensor isnt broken
     */
    void zeroSensorBroken(bool broke);

    void setHangIdleMode(bool enabled);

    private:
    ThunderSparkMax *winch;
    frc::Servo ratchet{PWM_HANG_RATCHET};

    frc::DigitalInput zeroSensor{DIO_HANG_RESET};
    bool lastSensorReading = false;

    HangMovement moveDirection = STOP;
    bool slowRetract = false;

    enum HangState {
        STOPPED,
        WINCH_DISENGAGE,
        MOVING_UP,
        MOVING_DOWN
    };
    HangState currentState = STOPPED;

    double moveDownRamp = 0;

    void setRatchetEngaged(bool engaged);
    bool ratchetEngaged = false;

    double preEngageEncoder = 0;

    bool zeroSensorBroke = false;
};
