#pragma once

#include "ThunderSparkMax.h"
#include "IOMap.h"
#include "Feedback.h"
#include "rev/CANSparkMax.h"
#include <frc/DigitalInput.h>
#include "Log.h"

class Intake {
    public:
        Intake();

        void process();
        void reset();
        void debug(Feedback* feedback);

        /*
        Sets the direction of the intake motors
        INTAKE spins the intake motor in a positive direction to intake power cells
        OUTTAKE spins the intake motor in a negative direction to outtake powercells
        NOTTAKE does not spin the motors
        */
        enum IntakeDir{
            INTAKE,
            OUTTAKE,
            NOTTAKE
        };

        /*
        Sets the position of the intake mechanism
        DEPLOY has the mechanism outside of FP, ready to intake balls
        RETRACT stows the intake mechanism inside of FP
        */
        enum IntakePos{
            DEPLOY,
            RETRACT
        };

        void handleLog(Log* log);

    private:
        ThunderSparkMax *intakeDeployMotor; //object creation for the motor that pivots the intake mechanism inside and outside of FP
        ThunderSparkMax *intakeSpinMotor; //object creation for the motor that spins the wheels on the intake mechanism to intake/outtake balls (uses the fake Spark Max class)

        void setIntakeDeployMotorSpeed (double speed); //sets the speed of the intake deploy motors
        void setIntakeSpinMotorSpeed(double speed); //sets the speed of the intake spin motors
        void zeroPivotEncoder(); // zeros the encoder with the encZeroSensor

        IntakePos intakePos; //objects for intakePos function
        IntakeDir intakeDir; //object for intakeDir function

        frc::DigitalInput encZeroSensor {DIO_INTAKE_RESET}; // object for the sensor to zero the encoder for the pivot motor

        bool isIntakeIn;
        bool startup;
        bool lastZero;

    public:
        void setIntakeDir(IntakeDir direction); //uses the enumerator to set the direction of the intake spin motors
        void setIntakePos(IntakePos position); //uses the enumerator to set the position of the intake mechanism
        void setPivotSensorBroken(bool broken); //toggled by the broken switch on the controls. if true, disregard the readings from the pivot sensor
        void setIntakeIdleMode(bool enabled); //sets the idle mode of the pivot motor depending on whether or not the robot is enabled or disabled

        bool intakePivBroken; 
};