#pragma once
#include "rev/CANSparkMax.h"

class ThunderSparkMaxImpl;

class ThunderSparkMax {
    public:
        ThunderSparkMax();
        virtual ~ThunderSparkMax();
        virtual void Set(double speed) = 0;
        virtual double Get() = 0;
        // Rate is amount of time to go from 0 to full throttle in seconds
        virtual void SetOpenLoopRampRate(double rate) = 0;
        
        // Returns rotations of encoder
        virtual double GetEncoder() = 0;
        virtual void SetEncoder(double rotations) = 0;

        enum IdleMode {BRAKE, COAST};
        virtual void SetIdleMode(IdleMode idleMode) = 0;
        virtual void SetInverted(bool inverted) = 0;
        virtual void Follow(ThunderSparkMax *leader, bool invertOutput = false) = 0;

        virtual rev::CANPIDController* getSparkPID() = 0;
        virtual rev::CANEncoder* getSparkEncoder() = 0;
        virtual double getSparkCurrent() = 0;

        enum MotorID {
            // NOTE: Implementation depends on the order of these!!
            // Do not reorder/add/remove anything from this without
            // also updating the implementation to match!!
            DriveFrontLeft = 0,
            DriveFrontRight,
            DriveRearLeft,
            DriveRearRight,
            IntakePivot,
            IntakeBeaterBars,
            StorageAgitator,
            StorageTransitionToShooter,
            ShooterPrimer, 
            ShooterLeft,
            ShooterRight,
            WheelOfFortune,
            Hang,
        };
        static ThunderSparkMax *create(MotorID id);
};

